@@ -89,11 +89,6 @@ void AsyncFizzClientT<SM>::connect(
     cachedPsk = fizzContext_->getPsk(*pskIdentity);
   }
 
-  auto echPolicy = fizzContext_->getECHPolicy();
-  if (!echConfigs && echPolicy && sni.hasValue()) {
-    echConfigs = echPolicy->getConfig(sni.value());
-  }
-
   fizzClient_.connect(
       fizzContext_,
       std::move(verifier),
@@ -662,8 +657,8 @@ template <typename SM>
 void AsyncFizzClientT<SM>::ActionMoveVisitor::operator()(
     SecretAvailable& secret) {
   fizz_probe_secret_available(
-      secret.secret.secret.size(),
       secret.secret.secret.data(),
+      secret.secret.secret.size(),
       KeyLogWriter::secretToNSSLabel(secret.secret.type)
           .value_or(std::numeric_limits<KeyLogWriter::Label>::max()),
       client_.getClientRandom()->data());
