@@ -17,6 +17,7 @@
 #include <fizz/protocol/Protocol.h>
 #include <fizz/protocol/StateMachine.h>
 #include <fizz/protocol/ech/Encryption.h>
+#include <fizz/protocol/ech/GreaseECH.h>
 #include <fizz/record/Extensions.h>
 
 using folly::Optional;
@@ -285,15 +286,22 @@ Actions handleError(
     newState.writeRecordLayer() = nullptr;
     newState.readRecordLayer() = nullptr;
   });
+
+  Actions actions;
+  actions.emplace_back(std::move(transition));
+
   if (alertDesc && state.writeRecordLayer()) {
-    Alert alert(*alertDesc);
-    WriteToSocket write;
-    write.contents.emplace_back(
-        state.writeRecordLayer()->writeAlert(std::move(alert)));
-    return actions(std::move(transition), std::move(write), std::move(error));
-  } else {
-    return actions(std::move(transition), std::move(error));
+    try {
+      Alert alert(*alertDesc);
+      WriteToSocket write;
+      write.contents.emplace_back(
+          state.writeRecordLayer()->writeAlert(std::move(alert)));
+      actions.emplace_back(std::move(write));
+    } catch (...) {
+    }
   }
+  actions.emplace_back(std::move(error));
+  return actions;
 }
 
 Actions handleAppCloseImmediate(const State& state) {
@@ -793,15 +801,10 @@ static ClientHello constructEncryptedClientHello(
   return chloOuter;
 }
 
-static std::unique_ptr<folly::IOBuf> randomIOBuf(
-    const Factory& factory,
-    size_t count) {
-  auto buf = folly::IOBuf::create(count);
-  if (count > 0) {
-    factory.makeRandomBytes(buf->writableData(), count);
-    buf->append(count);
-  }
-  return buf;
+static void checkContext(std::shared_ptr<const FizzClientContext>& context) {
+#if FIZZ_ENABLE_CONTEXT_COMPATIBILITY_CHECKS
+  context->validate();
+#endif
 }
 
 Actions
@@ -812,6 +815,8 @@ EventHandler<ClientTypes, StateEnum::Uninitialized, Event::Connect>::handle(
 
   auto context = std::move(connect.context);
 
+  checkContext(context);
+
   // Set up SNI (including possible replacement ECH SNI)
   folly::Optional<std::string> echSni;
   auto sni = std::move(connect.sni);
@@ -825,7 +830,8 @@ EventHandler<ClientTypes, StateEnum::Uninitialized, Event::Connect>::handle(
   // If we have a saved PSK, use the group to choose which groups to
   // send by default
   std::vector<NamedGroup> selectedShares;
-  if (psk && psk->group &&
+  if (context->getSendKeyShare() != SendKeyShare::AlwaysDefaultShares && psk &&
+      psk->group &&
       std::find(
           context->getSupportedGroups().begin(),
           context->getSupportedGroups().end(),
@@ -846,7 +852,7 @@ EventHandler<ClientTypes, StateEnum::Uninitialized, Event::Connect>::handle(
   Buf legacySessionId;
   if (context->getCompatibilityMode()) {
     constexpr size_t kRandomSize = Random().size();
-    legacySessionId = randomIOBuf(*context->getFactory(), kRandomSize);
+    legacySessionId = context->getFactory()->makeRandomIOBuf(kRandomSize);
   } else {
     legacySessionId = folly::IOBuf::create(0);
   }
@@ -884,11 +890,20 @@ EventHandler<ClientTypes, StateEnum::Uninitialized, Event::Connect>::handle(
     requestedExtensions.push_back(extension.extension_type);
   }
 
-  if (echParams.has_value() &&
-      echParams.value().supportedECHConfig.config.version ==
-          ech::ECHVersion::Draft15) {
-    ech::InnerECHClientHello chloIsInnerExt;
-    chlo.extensions.push_back(encodeExtension(std::move(chloIsInnerExt)));
+  if (echParams.has_value()) {
+    if (echParams.value().supportedECHConfig.config.version ==
+        ech::ECHVersion::Draft15) {
+      ech::InnerECHClientHello chloIsInnerExt;
+      chlo.extensions.push_back(encodeExtension(std::move(chloIsInnerExt)));
+      requestedExtensions.push_back(ExtensionType::encrypted_client_hello);
+    }
+  } else if (
+      context->getGreaseECHSetting().hasValue() &&
+      context->getGreaseECHSetting()->payloadStrategy ==
+          ech::PayloadGenerationStrategy::UniformRandom) {
+    auto greaseECH = ech::generateGreaseECH(
+        *context->getGreaseECHSetting(), *context->getFactory(), 0);
+    chlo.extensions.push_back(encodeExtension(std::move(greaseECH)));
     requestedExtensions.push_back(ExtensionType::encrypted_client_hello);
   }
 
@@ -945,8 +960,7 @@ EventHandler<ClientTypes, StateEnum::Uninitialized, Event::Connect>::handle(
   folly::Optional<ClientPresharedKey> greasePsk;
 
   if (echParams.has_value()) {
-    // Swap out randoms first.
-    echRandom = std::move(random);
+    echRandom = random;
     context->getFactory()->makeRandomBytes(random.data(), random.size());
 
     // Generate GREASE PSK (if needed)
@@ -971,6 +985,18 @@ EventHandler<ClientTypes, StateEnum::Uninitialized, Event::Connect>::handle(
 
     // Update the client hello with the ECH client hello outer
     encodedClientHello = encodeHandshake(chlo);
+  } else if (
+      context->getGreaseECHSetting().hasValue() &&
+      context->getGreaseECHSetting()->payloadStrategy ==
+          ech::PayloadGenerationStrategy::Computed) {
+    auto greaseECH = ech::generateGreaseECH(
+        *context->getGreaseECHSetting(),
+        *context->getFactory(),
+        encodedClientHello->computeChainDataLength());
+    chlo.extensions.push_back(encodeExtension(std::move(greaseECH)));
+    requestedExtensions.push_back(ExtensionType::encrypted_client_hello);
+    // Update the client hello with the grease ECH extension
+    encodedClientHello = encodeHandshake(chlo);
   }
 
   auto readRecordLayer = context->getFactory()->makePlaintextReadRecordLayer();
