@@ -20,6 +20,8 @@ namespace ech {
 
 namespace {
 
+static const size_t kGreasePSKIdentitySize = 16;
+
 std::unique_ptr<folly::IOBuf> makeClientHelloOuterForAad(
     const ClientHello& clientHelloOuter) {
   // Copy client hello outer
@@ -396,16 +398,6 @@ void setAcceptConfirmation(
 
 namespace {
 
-static std::unique_ptr<folly::IOBuf> randomIOBuf(
-    const Factory& factory,
-    size_t count) {
-  auto buf = folly::IOBuf::create(count);
-  if (count > 0) {
-    factory.makeRandomBytes(buf->writableData(), count);
-    buf->append(count);
-  }
-  return buf;
-}
 // GREASE PSKs are essentially the same size as the source PSK with the actual
 // contents of all fields replaced with random data. For the HRR case, the PSK
 // identity is preserved.
@@ -420,8 +412,8 @@ ClientPresharedKey generateGreasePskCommon(
     if (keepIdentity) {
       greaseIdentity.psk_identity = identity.psk_identity->clone();
     } else {
-      size_t identitySize = identity.psk_identity->computeChainDataLength();
-      greaseIdentity.psk_identity = randomIOBuf(*factory, identitySize);
+      greaseIdentity.psk_identity =
+          factory->makeRandomIOBuf(kGreasePSKIdentitySize);
     }
 
     factory->makeRandomBytes(
@@ -432,7 +424,7 @@ ClientPresharedKey generateGreasePskCommon(
     const auto& binder = source.binders.at(i);
     PskBinder greaseBinder;
     size_t binderSize = binder.binder->computeChainDataLength();
-    greaseBinder.binder = randomIOBuf(*factory, binderSize);
+    greaseBinder.binder = factory->makeRandomIOBuf(binderSize);
     grease.binders.push_back(std::move(greaseBinder));
   }
   return grease;
