@@ -286,15 +286,22 @@ Actions handleError(
     newState.writeRecordLayer() = nullptr;
     newState.readRecordLayer() = nullptr;
   });
+
+  Actions actions;
+  actions.emplace_back(std::move(transition));
+
   if (alertDesc && state.writeRecordLayer()) {
-    Alert alert(*alertDesc);
-    WriteToSocket write;
-    write.contents.emplace_back(
-        state.writeRecordLayer()->writeAlert(std::move(alert)));
-    return actions(std::move(transition), std::move(write), std::move(error));
-  } else {
-    return actions(std::move(transition), std::move(error));
+    try {
+      Alert alert(*alertDesc);
+      WriteToSocket write;
+      write.contents.emplace_back(
+          state.writeRecordLayer()->writeAlert(std::move(alert)));
+      actions.emplace_back(std::move(write));
+    } catch (...) {
+    }
   }
+  actions.emplace_back(std::move(error));
+  return actions;
 }
 
 Actions handleAppCloseImmediate(const State& state) {
@@ -419,6 +426,20 @@ EventHandler<ServerTypes, StateEnum::Uninitialized, Event::Accept>::handle(
       MutateState(&Transition<StateEnum::ExpectingClientHello>));
 }
 
+/**
+ * ECH may change the effective client hello we deal with. Some logging may
+ * actually want to observe fields on the ClientHelloOuter, which is why there
+ * is an explicitly separate logging hook for these cases.
+ */
+static void addHandshakeLoggingPreECH(
+    const State& state,
+    const ClientHello& chlo) {
+  auto logging = state.handshakeLogging();
+  if (logging) {
+    logging->populateFromPreECHClientHello(chlo);
+  }
+}
+
 static void addHandshakeLogging(const State& state, const ClientHello& chlo) {
   auto logging = state.handshakeLogging();
   if (!logging) {
@@ -556,7 +577,6 @@ static ResumptionStateResult getResumptionState(
     return ResumptionStateResult(
         std::make_pair(PskType::Rejected, folly::none));
   } else {
-    FOLLY_SDT(fizz, session_cache_ResumptionSuccess);
     const auto& ident = psks->identities[kPskIndex].psk_identity;
     return ResumptionStateResult(
         ticketCipher->decrypt(ident->clone()),
@@ -1206,6 +1226,7 @@ AsyncActions
 EventHandler<ServerTypes, StateEnum::ExpectingClientHello, Event::ClientHello>::
     handle(const State& state, Param& param) {
   ClientHello chlo = std::move(*param.asClientHello());
+  addHandshakeLoggingPreECH(state, chlo);
 
   auto cookieState = getCookieState(chlo, state.context()->getCookieCipher());
 
@@ -1258,6 +1279,12 @@ EventHandler<ServerTypes, StateEnum::ExpectingClientHello, Event::ClientHello>::
                            .hostname->to<std::string>();
       }
 
+      auto logging = state.handshakeLogging();
+      if (logging) {
+        fallback.handshakeLogging =
+            std::make_unique<HandshakeLogging>(*logging);
+      }
+
       return actions(
           MutateState(&Transition<StateEnum::Error>), std::move(fallback));
     } else {
@@ -1319,6 +1346,8 @@ EventHandler<ServerTypes, StateEnum::ExpectingClientHello, Event::ClientHello>::
             pskType = PskType::Rejected;
             pskMode = folly::none;
             resState = folly::none;
+          } else {
+            FOLLY_SDT(fizz, session_cache_ResumptionSuccess);
           }
         } else {
           pskMode = folly::none;
@@ -2220,6 +2249,10 @@ AsyncActions EventHandler<
     } else {
       newCert = std::move(leafCert);
     }
+  } catch (const FizzVerificationException& e) {
+    throw FizzVerificationException(
+        folly::to<std::string>("client certificate failure: ", e.what()),
+        e.getAlert());
   } catch (const FizzException&) {
     throw;
   } catch (const std::exception& e) {
