@@ -12,15 +12,15 @@
 namespace fizz {
 extern "C" {
 void fizz_probe_secret_available(
-    long unsigned int secretSize,
     unsigned char* secretData,
+    unsigned char secretSize,
     fizz::KeyLogWriter::Label nssLabel,
     unsigned char* clientRandom) {
   FOLLY_SDT(
       fizz,
       fizz_secret_available,
-      secretSize,
       secretData,
+      secretSize,
       nssLabel,
       clientRandom);
 }
